@@ -64,6 +64,7 @@ func (root *Root) closeFile() {
 	log.Printf("close file %s", root.Doc.FileName)
 }
 
+// reload reload a current document.
 func (root *Root) reload(m *Document) {
 	if m.preventReload {
 		root.setMessagef("cannot reload: %s", m.FileName)
@@ -74,16 +75,20 @@ func (root *Root) reload(m *Document) {
 		log.Printf("cannot reload: %s", err)
 		return
 	}
+	root.releaseEventBuffer()
+	// Reserve time to read.
 	time.Sleep(100 * time.Millisecond)
 }
 
-func (root *Root) watch() {
+// toggleWatch toggles watch mode.
+func (root *Root) toggleWatch() {
 	root.Doc.WatchMode = !root.Doc.WatchMode
 	if root.Doc.WatchMode {
 		root.watchStart()
 	}
 }
 
+// watchStart starts watch mode.
 func (root *Root) watchStart() {
 	m := root.Doc
 	m.WatchInterval = max(m.WatchInterval, 1)
@@ -134,7 +139,6 @@ func (root *Root) goLine(input string) {
 		root.setMessage(ErrInvalidNumber.Error())
 		return
 	}
-	log.Println(lN)
 	nTh, err := strconv.Atoi(inputs[1])
 	if err != nil {
 		root.setMessage(ErrInvalidNumber.Error())
@@ -222,7 +226,6 @@ func (root *Root) setHeader(input string) {
 
 	root.Doc.Header = num
 	root.setMessagef("Set header lines %d", num)
-	root.Doc.ClearCache()
 }
 
 // setSkipLines sets the number of lines to skip.
@@ -242,7 +245,6 @@ func (root *Root) setSkipLines(input string) {
 
 	root.Doc.SkipLines = num
 	root.setMessagef("Set skip lines %d", num)
-	root.Doc.ClearCache()
 }
 
 // suspend suspends the current screen display and runs the shell.
@@ -411,6 +413,6 @@ func (root *Root) prepareStartX() {
 func (root *Root) updateEndNum() {
 	root.debugMessage(fmt.Sprintf("Update EndNum:%d", root.Doc.BufEndNum()))
 	root.prepareStartX()
-	root.statusDraw()
+	root.drawStatus()
 	root.Screen.Sync()
 }
