@@ -4,6 +4,7 @@ import (
 	"context"
 	"fmt"
 	"io"
+	"io/fs"
 	"os"
 	"sync"
 	"sync/atomic"
@@ -117,6 +118,46 @@ func NewDocument() (*Document, error) {
 	return m, nil
 }
 
+// OpenDocument opens a file and returns a Document.
+func OpenDocument(fileName string) (*Document, error) {
+	fi, err := os.Stat(fileName)
+	if err != nil {
+		return nil, err
+	}
+	if fi.IsDir() {
+		return nil, fmt.Errorf("%s %w", fileName, ErrIsDirectory)
+	}
+
+	m, err := NewDocument()
+	if err != nil {
+		return nil, err
+	}
+
+	// named pipe.
+	if fi.Mode()&fs.ModeNamedPipe != 0 {
+		m.seekable = false
+	}
+
+	if err := m.ReadFile(fileName); err != nil {
+		return nil, err
+	}
+	return m, nil
+}
+
+// STDINDocument returns a Document that reads stdin.
+func STDINDocument() (*Document, error) {
+	m, err := NewDocument()
+	if err != nil {
+		return nil, err
+	}
+
+	m.seekable = false
+	if err := m.ReadFile(""); err != nil {
+		return nil, err
+	}
+	return m, nil
+}
+
 // GetLine returns one line from buffer.
 func (m *Document) GetLine(n int) string {
 	m.mu.Lock()
@@ -224,7 +265,42 @@ func (m *Document) getContentsStr(lN int, lc lineContents) (string, map[int]int)
 	return m.lastContentsStr, m.lastContentsMap
 }
 
-// fistLine is the first line that excludes the SkipLines and Header.
+// firstLine is the first line that excludes the SkipLines and Header.
 func (m *Document) firstLine() int {
 	return m.SkipLines + m.Header
 }
+
+// SearchLine searches the document and returns the matching line.
+func (m *Document) SearchLine(ctx context.Context, searcher Searcher, num int) (int, error) {
+	num = max(num, 0)
+
+	for n := num; n < m.BufEndNum(); n++ {
+		if searcher.Match(m.GetLine(n)) {
+			return n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+	}
+
+	return 0, ErrNotFound
+}
+
+// BackSearchLine does a backward search on the document and returns a matching line.
+func (m *Document) BackSearchLine(ctx context.Context, searcher Searcher, num int) (int, error) {
+	num = min(num, m.BufEndNum()-1)
+
+	for n := num; n >= 0; n-- {
+		if searcher.Match(m.GetLine(n)) {
+			return n, nil
+		}
+		select {
+		case <-ctx.Done():
+			return 0, ErrCancel
+		default:
+		}
+	}
+	return 0, ErrNotFound
+}
