@@ -12,7 +12,7 @@ import (
 )
 
 // UpdateInterval is the update interval that calls eventUpdate().
-var UpdateInterval time.Duration = 50
+var UpdateInterval = 50 * time.Millisecond
 
 // main is manages and executes events in the main routine.
 func (root *Root) main(ctx context.Context, quitChan chan<- struct{}) {
@@ -57,19 +57,19 @@ func (root *Root) main(ctx context.Context, quitChan chan<- struct{}) {
 		case *eventPaste:
 			root.getClipboard(ctx)
 		case *eventSearch:
-			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
-			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine()+1, searchMatch)
+			searcher := root.setSearcher(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine()+1, searcher)
 		case *eventBackSearch:
-			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
-			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine()-1, searchMatch)
+			searcher := root.setSearcher(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine()-1, searcher)
 		case *viewModeInput:
 			root.setViewMode(ev.value)
 		case *searchInput:
-			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
-			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine(), searchMatch)
+			searcher := root.setSearcher(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, true, root.Doc.topLN+root.Doc.firstLine(), searcher)
 		case *backSearchInput:
-			searchMatch := root.setSearch(root.input.value, root.CaseSensitive)
-			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine(), searchMatch)
+			searcher := root.setSearcher(root.input.value, root.CaseSensitive)
+			root.searchMove(ctx, false, root.Doc.topLN+root.Doc.firstLine(), searcher)
 		case *gotoInput:
 			root.goLine(ev.value)
 		case *headerInput:
@@ -211,7 +211,7 @@ func (root *Root) followAll() {
 
 // updateInterval calls eventUpdate at regular intervals.
 func (root *Root) updateInterval(ctx context.Context) {
-	timer := time.NewTicker(time.Millisecond * UpdateInterval)
+	timer := time.NewTicker(UpdateInterval)
 	for {
 		select {
 		case <-timer.C:
@@ -470,3 +470,10 @@ func (root *Root) Reload() {
 		log.Println(err)
 	}
 }
+
+// releaseEventBuffer will release all event buffers.
+func (root *Root) releaseEventBuffer() {
+	for root.HasPendingEvent() {
+		_ = root.Screen.PollEvent()
+	}
+}
