@@ -5,7 +5,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/fs"
 	"log"
 	"os"
 	"os/signal"
@@ -348,94 +347,61 @@ func NewConfig() Config {
 	}
 }
 
+// NewRoot returns the structure of the oviewer.
+// NewRoot is a simplified version that can be used externally.
+func NewRoot(r io.Reader) (*Root, error) {
+	m, err := NewDocument()
+	if err != nil {
+		return nil, err
+	}
+	if err := m.ReadReader(r); err != nil {
+		return nil, err
+	}
+	return NewOviewer(m)
+}
+
 // Open reads the file named of the argument and return the structure of oviewer.
 // If there is no file name, create Root from standard input.
 // If there is only one file name, create Root from that file,
 // but return an error if the open is an error.
 // If there is more than one file name, create Root from multiple files.
 func Open(fileNames ...string) (*Root, error) {
-	var root *Root
-	var err error
 	switch len(fileNames) {
 	case 0:
-		root, err = openSTDIN()
-		if err != nil {
-			return nil, err
-		}
+		return openSTDIN()
 	case 1:
-		m, err := openFile(fileNames[0])
-		if err != nil {
-			return nil, err
-		}
-		root, err = NewOviewer(m)
-		if err != nil {
-			return nil, err
-		}
+		return openFile(fileNames[0])
 	default:
-		root, err = openFiles(fileNames)
-		if err != nil {
-			return nil, err
-		}
+		return openFiles(fileNames)
 	}
-
-	return root, nil
 }
 
-// NewRoot returns the structure of the oviewer.
-// NewRoot is a simplified version that can be used externally.
-func NewRoot(read io.Reader) (*Root, error) {
-	m, err := NewDocument()
+// openSTDIN creates root with standard input.
+func openSTDIN() (*Root, error) {
+	m, err := STDINDocument()
 	if err != nil {
 		return nil, err
 	}
-	if err := m.ReadAll(read); err != nil {
-		return nil, err
-	}
 	return NewOviewer(m)
 }
 
-func openSTDIN() (*Root, error) {
-	docList := make([]*Document, 0, 1)
-	m, err := NewDocument()
+// openFile creates root in one file.
+// If there is only one file, an error will occur if the file fails to open.
+func openFile(fileName string) (*Root, error) {
+	m, err := OpenDocument(fileName)
 	if err != nil {
 		return nil, err
 	}
-
-	m.seekable = false
-	if err := m.ReadFile(""); err != nil {
-		return nil, err
-	}
-	docList = append(docList, m)
-	return NewOviewer(docList...)
-}
-
-func openFile(fileName string) (*Document, error) {
-	fi, err := os.Stat(fileName)
-	if err != nil {
-		return nil, err
-	}
-	if fi.IsDir() {
-		return nil, fmt.Errorf("%s %w", fileName, ErrIsDirectory)
-	}
-	m, err := NewDocument()
-	if err != nil {
-		return nil, err
-	}
-
-	if fi.Mode()&fs.ModeNamedPipe != 0 {
-		m.seekable = false
-	}
-	if err := m.ReadFile(fileName); err != nil {
-		return nil, err
-	}
-	return m, nil
+	return NewOviewer(m)
 }
 
+// openFiles opens multiple files and creates root.
+// It will continue even if there are files that fail to open.
 func openFiles(fileNames []string) (*Root, error) {
 	errors := make([]string, 0)
 	docList := make([]*Document, 0)
 	for _, fileName := range fileNames {
-		m, err := openFile(fileName)
+		m, err := OpenDocument(fileName)
 		if err != nil {
 			errors = append(errors, fmt.Sprintf("open error: %s", err))
 			continue
@@ -513,12 +479,9 @@ func (root *Root) setKeyConfig() (map[string][]string, error) {
 	return keyBind, nil
 }
 
+// SetKeyHandler assigns a new key handler.
 func (root *Root) SetKeyHandler(name string, keys []string, handler func()) error {
-	c := root.keyConfig
-	if err := setHandler(c, name, keys, handler); err != nil {
-		return err
-	}
-	return nil
+	return setHandler(root.keyConfig, name, keys, handler)
 }
 
 // Run starts the terminal pager.
@@ -548,13 +511,7 @@ func (root *Root) Run() error {
 		root.Screen.EnableMouse()
 	}
 
-	// Call from man command.
-	manPN := os.Getenv("MAN_PN")
-	if len(manPN) > 0 {
-		root.Doc.Caption = manPN
-		// Bug?? Clipboard fails when called by man.
-		root.Screen.DisableMouse()
-	}
+	root.optimizedMan()
 
 	for n, doc := range root.DocList {
 		log.Printf("open [%d]%s", n, doc.FileName)
@@ -563,14 +520,8 @@ func (root *Root) Run() error {
 			doc.WatchMode = true
 		}
 	}
-	root.Screen.Clear()
 
-	list := make([]string, 0, len(root.Config.Mode)+1)
-	list = append(list, "general")
-	for name := range root.Config.Mode {
-		list = append(list, name)
-	}
-	root.input.ModeCandidate.list = list
+	root.setModeConfig()
 
 	root.ViewSync()
 	// Exit if fits on screen
@@ -610,6 +561,27 @@ func (root *Root) Run() error {
 	}
 }
 
+func (root *Root) optimizedMan() {
+	// Call from man command.
+	manPN := os.Getenv("MAN_PN")
+	if len(manPN) == 0 {
+		return
+	}
+
+	root.Doc.Caption = manPN
+	// Bug?? Clipboard fails when called by man.
+	root.Screen.DisableMouse()
+}
+
+func (root *Root) setModeConfig() {
+	list := make([]string, 0, len(root.Config.Mode)+1)
+	list = append(list, "general")
+	for name := range root.Config.Mode {
+		list = append(list, name)
+	}
+	root.input.ModeCandidate.list = list
+}
+
 // Close closes the oviewer.
 func (root *Root) Close() {
 	root.Screen.Fini()
@@ -621,7 +593,7 @@ func (root *Root) setMessage(msg string) {
 	}
 	root.message = msg
 	root.debugMessage(msg)
-	root.statusDraw()
+	root.drawStatus()
 	root.Show()
 }
 
@@ -697,7 +669,7 @@ func (root *Root) prepareView() {
 	root.vHight = max(root.vHight, 1)
 
 	root.lnumber = make([]lineNumber, root.vHight+1)
-	root.statusPos = root.vHight - statusline
+	root.statusPos = root.vHight - statusLine
 }
 
 // docSmall returns with bool whether the file to display fits on the screen.
@@ -748,23 +720,3 @@ func (root *Root) WriteLog() {
 	end := m.BufEndNum()
 	m.Export(os.Stdout, start, end)
 }
-
-// leftMostX returns a list of left - most x positions when wrapping.
-func (root *Root) leftMostX(lN int) ([]int, error) {
-	lc, err := root.Doc.contentsLN(lN, root.Doc.TabWidth)
-	if err != nil {
-		return nil, err
-	}
-
-	listX := make([]int, 0, (len(lc)/root.vWidth)+1)
-	width := (root.vWidth - root.startX)
-
-	listX = append(listX, 0)
-	for n := width; n < len(lc); n += width {
-		if lc[n-1].width == 2 {
-			n--
-		}
-		listX = append(listX, n)
-	}
-	return listX, nil
-}
